import sys
import signal
from multiprocessing import Process
import typer
import requests
import os
import json
import shutil
from rich.progress import Progress, SpinnerColumn, TextColumn
from . import rest_helper as rest

app = typer.Typer()

def uid(p):
    print(p)
    return p['uid']

def project_names():
    r = requests.get(f'{rest.base_url}/api/bu/{rest.org}/project', headers=rest.headers)
    #sys.stderr.write(r.text)
    if r.status_code == 200:
        projects = r.json()
        names = map(lambda p: p['uid'], projects)
        return (list(names))
    else:
        sys.stderr.write(f"Could not list projects due to {r.status_code}\n")
        #os.kill(signal.SIGSTOP)
        raise typer.Exit(0)
        #return []

        #return map(list(r.json()), lambda e: e.uid)
#    return ["beamyhack"]


@app.command("list", help="Lists recordings in project")
def listRecordings(type: str = typer.Argument(default="all", help="all, processing, recent"),
         project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT',
                                     autocompletion=project_names)):

    #print(project)
    #raise typer.Exit()
    #try:
        #project_names()
    #except SystemExit as e:
    #    raise sys.exit(e)

    if project == "_":
        print("Something went wrong")
        raise typer.Exit()
    #else:
    #    print(project)

    if type == "all":
        rest.handle_get(f"/api/project/{project}/files/recording")
    elif type == "recent":
        rest.handle_get(f"/api/project/{project}/files/recording/recent")
    elif type == "processing":
        rest.handle_get(f"/api/project/{project}/files/recording/processing")
    else:
        print("Unknown type: " + type)


@app.command(help="Shows details about a specific recording in project")
def describe(name: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/files/recording/{name}")


def doStart(name: str, project: str, api_key: str, return_response: bool = False):
    if api_key == "":
        body = {"size": "S"}
    else:
        body = {"size": "S", 'apiKey': api_key}
    return rest.handle_post(f"/api/project/{project}/brokers/{name}", body=json.dumps(body),
                            return_response=return_response)


@app.command(help="Plays all recording files or a single recording")
def play(recording_session: str = typer.Option(..., help="Recording session id"),
         recording_file: str = typer.Option("", help="Recording file"),
         ensure_broker_started: bool = typer.Option(default=False, help="Ensure broker exists, start otherwise"),
         broker: str = typer.Option(..., help="Broker name to play on"),
         project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        v = progress.add_task(description=f"Verifying broker {broker} exists...", total=100)
        r = requests.get(f'{rest.base_url}/api/project/{project}/brokers/{broker}', headers=rest.headers)
        progress.update(v, advance=100.0)
        if r.status_code == 404:
            if ensure_broker_started:
                progress.add_task(description=f"Starting broker {broker}...", total=1)
                r = doStart(broker, project, '', return_response=True)
                if r.status_code != 200:
                    print(r.text)
                    exit(0)
            else:
                print("Broker not running, use --ensure-broker-started true to start the broker")
                exit(0)
        progress.add_task(
            description=f"Uploading recording {recording_session} to {broker} and setting play mode to pause...",
            total=None)
        if recording_file == "":
            rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}/upload",
                            params={'brokerName': broker})
        else:
            rest.handle_get(f"/api/project/{project}/files/recording/{recording_session}/{recording_file}/upload",
                            params={'brokerName': broker})


@app.command(help="Downloads the specified recording file")
def download(recording_session: str = typer.Option(..., help="Recording session id"),
             recording_file: str = typer.Option("", help="Recording file"),
             project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Downloading {recording_file}", total=None)

        # First request the download url from cloud. This is a public signed url that is valid
        # for a short period of time
        get_signed_url_resp = requests.get(
            f'{rest.base_url}/api/project/{project}/files/recording/{recording_session}/recording-file/{recording_file}',
            headers=rest.headers(), allow_redirects=True)
        if get_signed_url_resp.status_code == 200:

            # Next download the actual file
            download_resp = requests.get(url=get_signed_url_resp.json()["downloadUrl"], stream=True)
            if download_resp.status_code == 200:
                with open(recording_file, 'wb') as out_file:
                    shutil.copyfileobj(download_resp.raw, out_file)
                print(f"{recording_file} downloaded")
            else:
                sys.stderr.write(f"Got unexpected status {download_resp.status_code}\n")
        else:
            sys.stderr.write(f"Got unexpected status {get_signed_url_resp.status_code}\n")

# @app.command()
# def upload(file: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
# files = {'upload_file': open(file, 'rb')}
# values = {'DB': 'photcat', 'OUT': 'csv', 'SHORT': 'short'}
# rest.headers["content-type"] = "application/octet-stream"
# r = requests.get(f"{rest.base_url}/api/project/{project}/files/recording/upload/{file}", headers=rest.headers)
# print(r.status_code)
# print(r.text)

# curl -X PUT -H 'Content-Type: application/octet-stream' --upload-file docker-compose.yml 'https://storage.googleapis.com/beamylabs-fileuploads-dev/projects/beamyhack/recording/myrecording?X-Goog-Algorithm=GOOG4-RSA-SHA256&X-Goog-Credential=recordings-upload-account%40beamycloud-dev.iam.gserviceaccount.com%2F20220729%2Fauto%2Fstorage%2Fgoog4_request&X-Goog-Date=20220729T134012Z&X-Goog-Expires=3000&X-Goog-SignedHeaders=content-type%3Bhost&X-Goog-Signature=d1fa7639349d6453aebfce8814d6e5685af03952d07aa4e3cb0d44dba7cf5e572f684c8120dba17cbc7ea6a0ef5450542a3c745c65e04272b34265d0ddcf1b67e6f2b5bfa446264a62d77bd7faabf45ad6bd2aec5225f57004b0a31cfe0480cba063a3807d86346b1da99ecbae3f3e6da8f44f06396dfc1fdc6f89e475abdf969142cef6f369f03aff41000c8abb28aa82185246746fd6c16b6b381baa2d586382a3d3067b6376ddba2b55b2b6f9d942913a1cbfbc61491ba6a615d7d5a0d9a476c357431143e9cea1411dfad9f01b1e1176dc8c056cbf08cccfd401a55d63c19d038f3ab42b712abc48d759047ac07862c4fae937c341e19b568bb60a4e4086'



import typer
import json
from rich.progress import Progress, SpinnerColumn, TextColumn
from . import rest_helper as rest

app = typer.Typer()


@app.command("list", help="Lists brokers in project")
def list(project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/brokers")


@app.command("describe", help="Shows details about a specific broker")
def describe(name: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    rest.handle_get(f"/api/project/{project}/brokers/{name}")


@app.command("stop", help="Stops and removes a broker from project")
def stop(name: str, project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT')):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Stopping broker {name}...", total=None)
        rest.handle_delete(f"/api/project/{project}/brokers/{name}")


def do_start(name: str, project: str, api_key: str, tag: str, return_response: bool = False):
    if tag == "":
        tag_to_use = None
    else:
        tag_to_use = tag

    if api_key == "":
        body = {"size": "S", 'tag': tag_to_use}
    else:
        body = {"size": "S", 'apiKey': api_key, 'tag': tag_to_use}
    return rest.handle_post(f"/api/project/{project}/brokers/{name}", body=json.dumps(body),
                            return_response=return_response)


@app.command(help="Starts a broker in project")
def start(name: str,
          project: str = typer.Option(..., help="Project ID", envvar='REMOTIVE_CLOUD_PROJECT'),
          tag: str = typer.Option("", help="Optional specific tag/version"),
          silent: bool = typer.Option(False, help="Optional specific tag/version"),
          api_key: str = typer.Option("", help="Start with your own api-key")):

    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
    ) as progress:
        progress.add_task(description=f"Starting broker {name}...", total=None)
        do_start(name, project, api_key, tag, return_response=silent)



import typer
from .brokers import app as broker_app
from .cloud.cloud_cli import app as cloud_app

# from cloud import cloud_cli

app = typer.Typer()


# def run_cli():

app.add_typer(broker_app, name="broker", help="Manage a single broker - local or cloud")
app.add_typer(cloud_app, name="cloud", help="Manage resources in RemotiveCloud", )



import time

import typer
import json
from rich.progress import Progress, SpinnerColumn, TextColumn
from . import recordings, brokers, configs, auth, service_accounts
from . import rest_helper as rest

app = typer.Typer()


@app.command(help="Who am I?")
def whoami():
    rest.handle_get('/api/whoami')


@app.command(help="List licenses for an organisation")
def licenses(
        organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION'),
        filter: str = typer.Option("all", help="all, valid, expired")
):
    rest.handle_get(f"/api/bu/{organisation}/licenses", {'filter': filter})


@app.command(help="Get your organisation and projects with permissions")
def home(
        organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION'),
):
    rest.handle_get(f"/api/bu/{organisation}/me")


# @app.command(help="Get your organisation and projects with permissions")
# def home2():
#    rest.handle_get(f"/api/home")


@app.command(help="List users in an organisation")
def users(
        organisation: str = typer.Option(..., help="Organisation ID", envvar='REMOTIVE_CLOUD_ORGANISATION'),
):
    rest.handle_get(f"/api/bu/{organisation}/users")


app.add_typer(brokers.app, name="brokers", help="Manage cloud broker lifecycle")
app.add_typer(recordings.app, name="recordings", help="Manage recordings")
app.add_typer(configs.app, name="configs", help="Manage signal databases")
app.add_typer(auth.app, name="auth", help="Manage access to cloud resources")
app.add_typer(service_accounts.app, name="service-accounts", help="Manage project service account keys")


if __name__ == "__main__":
    app()



import json
import time
import typer
#import network
from .lib.broker import Broker
#from network import MDNS


import argparse
import logging
from time import sleep
from typing import cast

from zeroconf import (
    IPVersion,
    ServiceBrowser,
    ServiceStateChange,
    Zeroconf,
    ZeroconfServiceTypes,
)

app = typer.Typer()




@app.command(help="List signals on broker")
def signals(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY", envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    print("Listing available signals")
    available_signals = broker.list_signal_names()
    print(json.dumps(available_signals))


@app.command(help="List namespaces on broker")
def namespaces(
        url: str = typer.Option(..., help="Broker URL", envvar='REMOTIVE_BROKER_URL'),
        api_key: str = typer.Option("offline", help="Cloud Broker API-KEY", envvar='REMOTIVE_BROKER_API_KEY')
):
    broker = Broker(url, api_key)
    namespaces = broker.list_namespaces()
    print(json.dumps(namespaces))

@app.command(help="Discover brokers on this network")
def discover():
    #print("Not implemented")

    zeroconf = Zeroconf(ip_version=IPVersion.V4Only)

    services = ["_remotivebroker._tcp.local.","_googlecast._tcp.local."]
    #services = list(ZeroconfServiceTypes.find(zc=zeroconf))

    print("\nLooking for RemotiveBrokers on your network, press Ctrl-C to exit...\n")
    browser = ServiceBrowser(zeroconf, services, handlers=[on_service_state_change])

    try:
        while True:
            sleep(0.1)
    except KeyboardInterrupt:
        pass
    finally:
        zeroconf.close()

    #network.MDNS.init()

    # Perform a query for 2000 ms
    #q = MDNS.query(2000, "broker", MDNS.PROTO_TCP)

    # Print out the query's result
    #if q is not None:
    #    for elem in q:
    #        print(elem.instance_name())
    #        print(elem.hostname())
    #        print(elem.addr())
    #        print(elem.port())
    #        print(elem.txt())


def on_service_state_change(
        zeroconf: Zeroconf, service_type: str, name: str, state_change: ServiceStateChange
) -> None:
    #print(f"Service {name} state changed: {state_change}")

    if state_change is ServiceStateChange.Removed:
        print(f"Service {name} was removed")

    if state_change is ServiceStateChange.Updated:
        print(f"Service {name} was updated")

    if state_change is ServiceStateChange.Added:
        print(f"Discovered {name} ")
        info = zeroconf.get_service_info(service_type, name)
        #print("Info from zeroconf.get_service_info: %r" % (info))

        if info:
            #addresses = ["%s:%d" % (addr, cast(int, info.port)) for addr in info.parsed_scoped_addresses()]
            for addr in info.parsed_scoped_addresses():
                print(addr)
            #print("  Weight: %d, priority: %d" % (info.weight, info.priority))
            #print(f"  Server: {info.server}")
            #if info.properties:
            #    print("  Properties are:")
            #    for key, value in info.properties.items():
            #        print(f"    {key}: {value}")
            #else:
            #    print("  No properties")
        else:
            print("  No info")
        print('\n')

if __name__ == "__main__":
    app()



import binascii
import queue
from threading import Thread
import remotivelabs.broker.sync as br


class Broker:

    def __init__(self, url, api_key):
        self.url = url
        self.api_key = api_key
        self.q = queue.Queue()
        """Main function, checking arguments passed to script, setting up stubs, configuration and starting Threads."""
        # Setting up stubs and configuration
        self.intercept_channel = br.create_channel(url, api_key)

        self.network_stub = br.network_api_pb2_grpc.NetworkServiceStub(self.intercept_channel)
        self.system_stub = br.system_api_pb2_grpc.SystemServiceStub(self.intercept_channel)
        self.traffic_stub = br.traffic_api_pb2_grpc.TrafficServiceStub(self.intercept_channel)
        self.signal_creator = br.SignalCreator(self.system_stub)

    def play(self, namespace: str, path: str):
        playback_list = [
            {
                "namespace": namespace,
                "path": path,
                "mode": br.traffic_api_pb2.Mode.PLAY,
            }]

        status = self.traffic_stub.PlayTraffic(
            br.traffic_api_pb2.PlaybackInfos(
                playbackInfo=list(map(self.__create_playback_config, playback_list))
            )
        )

    def list_namespaces(self):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())
        namespaces = []
        for networkInfo in configuration.networkInfo:
            namespaces.append(networkInfo.namespace.name)
        return namespaces

    def list_signal_names(self, namespace):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())

        signal_names = []
        for networkInfo in configuration.networkInfo:
            res = self.system_stub.ListSignals(networkInfo.namespace)
            for finfo in res.frame:
                for sinfo in finfo.childInfo:
                    signal_names.append({'signal': sinfo.id.name, 'namespace' : networkInfo.namespace.name})
        return signal_names

    def list_signal_names(self):
        # Lists available signals
        configuration = self.system_stub.GetConfiguration(br.common_pb2.Empty())

        signal_names = []
        for networkInfo in configuration.networkInfo:
            res = self.system_stub.ListSignals(networkInfo.namespace)
            for finfo in res.frame:
                for sinfo in finfo.childInfo:
                    signal_names.append({'signal': sinfo.id.name, 'namespace' : networkInfo.namespace.name})
        return signal_names

    def subscribe(self, signals: list, on_frame, changed_values_only: bool = True):
        client_id = br.common_pb2.ClientId(id="cloud_demo")

        signals_to_subscribe_on = \
            map(lambda signal: self.signal_creator.signal(signal, "custom_can"), signals)

        Thread(
            target=br.act_on_signal,
            args=(
                client_id,
                self.network_stub,
                signals_to_subscribe_on,
                changed_values_only,  # True: only report when signal changes
                lambda frame: self.__each_signal(frame, on_frame),
                lambda sub: (self.q.put(("cloud_demo", sub))),
            ),
        ).start()
        # Wait for subscription
        ecu, subscription = self.q.get()
        return subscription

    @classmethod
    def connect(cls, url, api_key):
        return Broker(url, api_key)

    def __each_signal(self, signals, callback):
        callback(map(lambda s: {
            'timestamp_nanos': s.timestamp,
            'name': s.id.name,
            'value': self.__get_value(s)
        }, signals))

    @staticmethod
    def __get_value(signal):
        if signal.raw != b"":
            return "0x" + binascii.hexlify(signal.raw).decode("ascii")
        elif signal.HasField("integer"):
            return signal.integer
        elif signal.HasField("double"):
            return signal.double
        elif signal.HasField("arbitration"):
            return signal.arbitration
        else:
            return "empty"

    @staticmethod
    def __create_playback_config(item):
        """Creating configuration for playback

        Parameters
        ----------
        item : dict
            Dictionary containing 'path', 'namespace' and 'mode'

        Returns
        -------
        PlaybackInfo
            Object instance of class

        """
        playbackConfig = br.traffic_api_pb2.PlaybackConfig(
            fileDescription=br.system_api_pb2.FileDescription(path=item["path"]),
            namespace=br.common_pb2.NameSpace(name=item["namespace"]),
        )
        return br.traffic_api_pb2.PlaybackInfo(
            playbackConfig=playbackConfig,
            playbackMode=br.traffic_api_pb2.PlaybackMode(mode=item["mode"]),
        )


Broker.connect = classmethod(Broker.connect)
